==> layout-7.1.18-sources.jar: com/itextpdf/layout/ColumnDocumentRenderer.java <==
/*

    This file is part of the iText (R) project.
    Copyright (c) 1998-2022 iText Group NV
    Authors: Bruno Lowagie, Paulo Soares, et al.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License version 3
    as published by the Free Software Foundation with the addition of the
    following permission added to Section 15 as permitted in Section 7(a):
    FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY
    ITEXT GROUP. ITEXT GROUP DISCLAIMS THE WARRANTY OF NON INFRINGEMENT
    OF THIRD PARTY RIGHTS

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU Affero General Public License for more details.
    You should have received a copy of the GNU Affero General Public License
    along with this program; if not, see http://www.gnu.org/licenses or write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA, 02110-1301 USA, or download the license from the following URL:
    http://itextpdf.com/terms-of-use/

    The interactive user interfaces in modified source and object code versions
    of this program must display Appropriate Legal Notices, as required under
    Section 5 of the GNU Affero General Public License.

    In accordance with Section 7(b) of the GNU Affero General Public License,
    a covered work must retain the producer line in every PDF that is created
    or manipulated using iText.

    You can be released from the requirements of the license by purchasing
    a commercial license. Buying such a license is mandatory as soon as you
    develop commercial activities involving the iText software without
    disclosing the source code of your own applications.
    These activities include: offering paid services to customers as an ASP,
    serving PDFs on the fly in a web application, shipping iText with a closed
    source product.

    For more information, please contact iText Software Corp. at this
    address: sales@itextpdf.com
 */
package com.itextpdf.layout;

import com.itextpdf.kernel.geom.Rectangle;
import com.itextpdf.layout.layout.LayoutArea;
import com.itextpdf.layout.layout.LayoutResult;
import com.itextpdf.layout.layout.RootLayoutArea;
import com.itextpdf.layout.property.AreaBreakType;
import com.itextpdf.layout.renderer.DocumentRenderer;
import com.itextpdf.layout.renderer.IRenderer;

/**
 * This class is used for convenient multi-column Document Layouting
 */
public class ColumnDocumentRenderer extends DocumentRenderer {

    protected Rectangle[] columns;
    protected int nextAreaNumber;

    /**
     * Creates a ColumnDocumentRenderer. Sets {@link #immediateFlush} to true.
     * 
     * @param document the {@link Document} on which this Renderer will calculate
     * and execute element placements
     * @param columns an array of {@link Rectangle} specifying the acceptable
     * positions for elements on a page
     */
    public ColumnDocumentRenderer(Document document, Rectangle[] columns) {
        super(document);
        this.columns = columns;
    }

    /**
     * Creates a ColumnDocumentRenderer whose elements need not be flushed
     * immediately.
     * 
     * @param document the {@link Document} on which this Renderer will calculate
     * and execute element placements
     * @param immediateFlush whether or not to flush contents as soon as possible
     * @param columns an array of {@link Rectangle} specifying the acceptable
     * positions for elements on a page
     */
    public ColumnDocumentRenderer(Document document, boolean immediateFlush, Rectangle[] columns) {
        super(document, immediateFlush);
        this.columns = columns;
    }

    /**
     * Gets the array index of the next area that will be written on after the
     * current one is full (overflowed).
     * @return the array index of the next area that will be written on
     */
    public int getNextAreaNumber() {
        return nextAreaNumber;
    }

    @Override
    public IRenderer getNextRenderer() {
        return new ColumnDocumentRenderer(document, immediateFlush, columns);
    }

    @Override
    protected LayoutArea updateCurrentArea(LayoutResult overflowResult) {
        if (overflowResult != null && overflowResult.getAreaBreak() != null && overflowResult.getAreaBreak().getType() != AreaBreakType.NEXT_AREA) {
            nextAreaNumber = 0;
        }
        if (nextAreaNumber % columns.length == 0) {
            super.updateCurrentArea(overflowResult);
        }
        return (currentArea = new RootLayoutArea(currentPageNumber, columns[nextAreaNumber++ % columns.length].clone()));
    }
}

==> layout-7.1.19-sources.jar: com/itextpdf/layout/ColumnDocumentRenderer.java <==
/*
    This file is part of the iText (R) project.
    Copyright (c) 1998-2023 Apryse Group NV
    Authors: Apryse Software.

    This program is offered under a commercial and under the AGPL license.
    For commercial licensing, contact us at https://itextpdf.com/sales.  For AGPL licensing, see below.

    AGPL licensing:
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.itextpdf.layout;

import com.itextpdf.kernel.geom.Rectangle;
import com.itextpdf.layout.layout.LayoutArea;
import com.itextpdf.layout.layout.LayoutResult;
import com.itextpdf.layout.layout.RootLayoutArea;
import com.itextpdf.layout.property.AreaBreakType;
import com.itextpdf.layout.renderer.DocumentRenderer;
import com.itextpdf.layout.renderer.IRenderer;

/**
 * This class is used for convenient multi-column Document Layouting
 */
public class ColumnDocumentRenderer extends DocumentRenderer {

    protected Rectangle[] columns;
    protected int nextAreaNumber;

    /**
     * Creates a ColumnDocumentRenderer. Sets {@link #immediateFlush} to true.
     * 
     * @param document the {@link Document} on which this Renderer will calculate
     * and execute element placements
     * @param columns an array of {@link Rectangle} specifying the acceptable
     * positions for elements on a page
     */
    public ColumnDocumentRenderer(Document document, Rectangle[] columns) {
        super(document);
        this.columns = columns;
    }

    /**
     * Creates a ColumnDocumentRenderer whose elements need not be flushed
     * immediately.
     * 
     * @param document the {@link Document} on which this Renderer will calculate
     * and execute element placements
     * @param immediateFlush whether or not to flush contents as soon as possible
     * @param columns an array of {@link Rectangle} specifying the acceptable
     * positions for elements on a page
     */
    public ColumnDocumentRenderer(Document document, boolean immediateFlush, Rectangle[] columns) {
        super(document, immediateFlush);
        this.columns = columns;
    }

    /**
     * Gets the array index of the next area that will be written on after the
     * current one is full (overflowed).
     * @return the array index of the next area that will be written on
     */
    public int getNextAreaNumber() {
        return nextAreaNumber;
    }

    @Override
    public IRenderer getNextRenderer() {
        return new ColumnDocumentRenderer(document, immediateFlush, columns);
    }

    @Override
    protected LayoutArea updateCurrentArea(LayoutResult overflowResult) {
        if (overflowResult != null && overflowResult.getAreaBreak() != null && overflowResult.getAreaBreak().getType() != AreaBreakType.NEXT_AREA) {
            nextAreaNumber = 0;
        }
        if (nextAreaNumber % columns.length == 0) {
            super.updateCurrentArea(overflowResult);
        }
        return (currentArea = new RootLayoutArea(currentPageNumber, columns[nextAreaNumber++ % columns.length].clone()));
    }
}
